/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// Calculate the following parameters:
// 1. End-to-end delay
// 2. Transmission delay
// 3. Round trip time
// 4. Throughput


#include "ns3/applications-module.h"
#include "ns3/core-module.h"
#include "ns3/internet-module.h"
#include "ns3/network-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/netanim-module.h"
#include "ns3/mobility-helper.h"
#include "ns3/mobility-model.h" 

// Default Network Topology
//
//       10.1.1.0
// n0 -------------- n1
//    point-to-point
//

using namespace ns3;

NS_LOG_COMPONENT_DEFINE("FirstScriptExample");

// Declare variables for the parameters
double node0_trans_time, node1_rec_time, node0_rec_time, round_trip_time = 0.0, total_delay = 0.0, prop_delay = 6560.0 ,throughput = 0.0, trans_delay = 0.0;
int pkts = 10;
int round_cnt = 0;
int pkt_cnt = 0;
static void
RxTraceClient(std::string context, Ptr<const Packet> p)
{
    //NS_LOG_UNCOND("Cleint received packet at " << Simulator::Now().GetSeconds() << " seconds");
    node0_rec_time = Simulator::Now().GetSeconds();
    round_trip_time += node0_rec_time - node0_trans_time;
    round_cnt++;
    if (round_cnt == pkts) {
    	NS_LOG_UNCOND("Round Trip Time: " << (round_trip_time*1000)/pkts << " ms");
    	round_cnt++;
    }
    
}

static void
TxTraceClient(std::string context,Ptr<const Packet> p)
{
    //NS_LOG_UNCOND("Client transmitted packet at " << Simulator::Now().GetSeconds() << " seconds");
    node0_trans_time = Simulator::Now().GetSeconds();
}

static void
RxTraceServer(std::string context, Ptr<const Packet> p)
{
    //NS_LOG_UNCOND("Server received packet at " << Simulator::Now().GetSeconds() << " seconds");
    node1_rec_time = Simulator::Now().GetSeconds();
    total_delay += (node1_rec_time - node0_trans_time)*1000;
    trans_delay += ((node1_rec_time - node0_trans_time)*1000 - prop_delay/1000000);
    pkt_cnt++;
    if (pkt_cnt == pkts) {
    	NS_LOG_UNCOND("End-to-end delay: " << total_delay/pkts << " ms");
    	NS_LOG_UNCOND("Transmission delay: " << trans_delay/pkts << " ms");
	NS_LOG_UNCOND("Throughput: " << total_delay/(p->GetSize()*pkts) << " bits/sec");
	pkt_cnt++;
    }
}

int main(int argc, char* argv[])
{
    CommandLine cmd(__FILE__);
    cmd.Parse(argc, argv);

    Time::SetResolution(Time::NS);
    //LogComponentEnable("UdpEchoClientApplication", LOG_LEVEL_INFO);
    //LogComponentEnable("UdpEchoServerApplication", LOG_LEVEL_INFO);
    
    // Create a node container with two nodes
    NodeContainer nodes;
    nodes.Create(2);
	
    // set the parameters for the point-to-point network	
    PointToPointHelper pointToPoint;
    pointToPoint.SetDeviceAttribute("DataRate", StringValue("100Mbps"));
    pointToPoint.SetChannelAttribute("Delay", TimeValue(NanoSeconds(6560)));
    
    // install net devices to the nodes
    NetDeviceContainer devices;
    devices = pointToPoint.Install(nodes);

    // install internet stack on the nodes
    InternetStackHelper stack;
    stack.Install(nodes);
    
    // Set IP address
    Ipv4AddressHelper address;
    address.SetBase("10.1.1.0", "255.255.255.0");
    
    // assign the IP address to the nodes
    Ipv4InterfaceContainer interfaces = address.Assign(devices);
 
    
    UdpEchoServerHelper echoServer(9);

    ApplicationContainer serverApps = echoServer.Install(nodes.Get(1));
    serverApps.Start(Seconds(1.0));
    serverApps.Stop(Seconds(10.0));

    UdpEchoClientHelper echoClient(interfaces.GetAddress(1), 9);
    echoClient.SetAttribute("MaxPackets", UintegerValue(10));
    echoClient.SetAttribute("Interval", TimeValue(Seconds(0.5)));
    echoClient.SetAttribute("PacketSize", UintegerValue(1024));

    ApplicationContainer clientApps = echoClient.Install(nodes.Get(0));
    clientApps.Start(Seconds(2.0));
    clientApps.Stop(Seconds(10.0));
    
    
    // Round trip time: Time between transmission and receiving times for node 0
    
    // Connect with Rx and Tx traces of UdpEchoServer and UdpEchoClient
    Config::Connect("/NodeList/0/ApplicationList/0/$ns3::UdpEchoClient/Tx", MakeCallback(&TxTraceClient));
    Config::Connect("/NodeList/0/ApplicationList/0/$ns3::UdpEchoClient/Rx", MakeCallback(&RxTraceClient));
    
    // For server
    Config::Connect("/NodeList/1/ApplicationList/0/$ns3::UdpEchoServer/Rx", MakeCallback(&RxTraceServer));
    MobilityHelper mobility;
    mobility.SetMobilityModel("ns3::ConstantPositionMobilityModel");
    mobility.Install(nodes);
    // For netanim animation
    AnimationInterface anim ("network_performance.xml");
    anim.SetConstantPosition(nodes.Get(0),0,150);
    anim.SetConstantPosition(nodes.Get(1),100,150);	
    Simulator::Run();
    Simulator::Destroy();
    return 0;
}

